<?php

namespace Drupal\commerce_boxnow;

use Drupal\commerce_shipping\Entity\ShippingMethod;
use Drupal\Core\Config\ConfigFactoryInterface;
use Drupal\Core\Database\Connection;
use Drupal\Core\Database\Database;
use Drupal\Core\DependencyInjection\DependencySerializationTrait;
use Drupal\Core\Entity\EntityTypeManagerInterface;
use Drupal\Core\Logger\LoggerChannelFactory;
use Drupal\Core\Session\AccountInterface;
use GuzzleHttp\ClientInterface;
use GuzzleHttp\Exception\RequestException;
use GuzzleHttp\Psr7\Request;
use Illuminate\Http\RedirectResponse;
use JsonException;

class CommerceBoxNowService {
  use DependencySerializationTrait;

  public const AUTHENDICATION_PATH = '/api/v1/auth-sessions';
  public const REQUEST_DELIVERY_PATH = '/api/v1/delivery-requests';

  /**
   * Constructs a GsisIdikaWebservice object.
   *
   * @param \GuzzleHttp\ClientInterface $client
   *   The HTTP client.
   * @param \Drupal\Core\Session\AccountInterface $currentUser
   *   The current_user service.
   * @param \Drupal\Core\Config\ConfigFactoryInterface $configFactory
   *   The "config.factory" service.
   * @param \Drupal\Core\Logger\LoggerChannelFactory $loggerChannelFactory
   *   The "logger.factory" service.
   * @param \Drupal\Core\Entity\EntityTypeManagerInterface $entityTypeManager
   *   The "entity_type.manager" service.
   * @param \Drupal\Core\Database\Connection $connection
   *   The database service.
   */
  public function __construct(
    protected ClientInterface $client,
    protected AccountInterface $currentUser,
    protected ConfigFactoryInterface $configFactory,
    protected LoggerChannelFactory $loggerChannelFactory,
    protected EntityTypeManagerInterface $entityTypeManager,
    protected Connection $connection,
  ) {}

  /**
   * Refreshes the API token if expired or missing.
   *
   * This function makes a request to the BoxNow authentication endpoint
   * to obtain a new access token and saves it to Drupal's state system.
   *
   * @return string|bool
   *   The refreshed access token if successful, or FALSE if the request fails.
   */
  protected function refreshToken() {
    $boxnow_config = $this->getBoxNowConfiguration();
    if (empty($boxnow_config)) {
      $this->loggerChannelFactory->get('commerce_boxnow')->error('BOX NOW configuration not found.');
      return FALSE;
    }
    $api_url = $boxnow_config['api_url'];
    $oauth_client_id = $boxnow_config['client_id'];
    $oauth_client_secret = $boxnow_config['client_secret'];

    // Prepare the body content
    try {
      $body = json_encode([
        'grant_type' => 'client_credentials',
        'client_id' => $oauth_client_id,
        'client_secret' => $oauth_client_secret,
      ], JSON_THROW_ON_ERROR);
    } catch (JsonException $e) {
      // Handle the exception if JSON encoding fails
      $this->loggerChannelFactory->get('commerce_boxnow')->error('Error encoding JSON: ' . $e->getMessage());
      return FALSE;
    }

    // Make the API request
    try {
      $headers = [
        'Content-Type' => 'application/json',
      ];
      $url = $api_url . self::AUTHENDICATION_PATH;
      $request = new Request('POST', $url, $headers, $body);
      $res = $this->client->sendAsync($request)->wait();
      // Decode JSON response
      $responseBody = $res->getBody()->getContents();
      if (empty($responseBody)) {
        $this->loggerChannelFactory->get('commerce_boxnow')->error('Empty response body from API. URL: @url.', [
          '@url' => self::AUTHENDICATION_PATH,
        ]);
        return FALSE;
      }
      $responseData = json_decode($responseBody, TRUE);
      // Check if decoding was successful
      if (json_last_error() !== JSON_ERROR_NONE) {
        $this->loggerChannelFactory->get('commerce_boxnow')->error('Error decoding JSON response: ' . json_last_error_msg());
        return FALSE;
      }
      $this->loggerChannelFactory->get('commerce_boxnow')->info('Response Data: ' . print_r($responseData, TRUE));

      // Extract token and expiration and save them in config.
      if (isset($responseData['access_token'], $responseData['expires_in'])) {
        $expiration = time() + $responseData['expires_in'] - 60; // 1-minute buffer.
        // Store the token in the state
        \Drupal::state()->set('boxnow_api_token', [
          'token' => $responseData['access_token'],
          'expiration' => $expiration,
        ]);
        $this->loggerChannelFactory->get('commerce_boxnow')->info('Token successfully refreshed and saved in state.');
        return $responseData['access_token'];
      }
    }
    catch (RequestException $e) {
      $this->loggerChannelFactory->get('commerce_boxnow')->error('Request to BoxNow failed. URL: @url, Error: @error', [
        '@url' => self::AUTHENDICATION_PATH,
        '@error' => $e->getMessage(),
      ]);
      return FALSE;
    }

    return FALSE;
  }

  /**
   * Retrieves the current API token.
   *
   * This function retrieves a stored token if it is still valid.
   * If the token is missing or expired, it will attempt to refresh it.
   *
   * @return string|bool
   *   The access token if valid or successfully refreshed, or FALSE on failure.
   */
  public function getToken() {
    // Retrieve the stored token and expiration
    $stored_data = \Drupal::state()->get('boxnow_api_token');

    if ($stored_data
      && isset($stored_data['token'], $stored_data['expiration'])
      && $stored_data['expiration'] > time()) {
      // Token is still valid, return it
      return $stored_data['token'];
    }
    // If no valid token or token is expired, refresh it
    $new_token = $this->refreshToken();
    if ($new_token) {
      return $new_token;
    }
    // If refresh fails, return FALSE or handle accordingly
    return FALSE;
  }

  /**
   * Sends a delivery request to the BoxNow API.
   *
   * This function prepares the delivery request payload using order details
   * and sends it to the BoxNow API to initiate a delivery. Requires a valid
   * access token and order data.
   *
   * @param array $order
   *   An associative array containing order details, including:
   *   - 'id': The order ID.
   *   - 'invoice_value': The value of the invoice.
   *   - 'phone': The contact phone number for delivery.
   *   - 'email': The contact email for delivery.
   *   - 'name': The contact name for delivery.
   *   - 'destination_locationId': The destination location ID.
   *   - 'total_weight': The total weight of the items.
   *
   * @return array|bool
   *   An associative array containing response details from the API, or
   *   FALSE if the request fails.
   */
  public function requestDelivery($order) {
    $token = $this->getToken();
    if (!$token) {
      $this->loggerChannelFactory->get('commerce_boxnow')->error('Failed to retrieve a valid API token.');
      return FALSE;
    }

    $boxnow_config = $this->getBoxNowConfiguration();
    if (empty($boxnow_config)) {
      $this->loggerChannelFactory->get('commerce_boxnow')->error('BOX NOW configuration not found.');
      return FALSE;
    }
    $api_url = $boxnow_config['api_url'];
    $notify_on_accepted = $boxnow_config['notify_on_accepted'];
    $contact_number = $boxnow_config['contact_number'];
    $contact_email = $boxnow_config['contact_email'];
    $contact_name = $boxnow_config['contact_name'];

    // Prepare the body content
    try {
      $body = json_encode([
        "orderNumber"=> $order['number'],
        "invoiceValue"=> $order['invoice_value'],
        "paymentMode"=> 'prepaid',
        "amountToBeCollected"=> "0.00",
        "notifyOnAccepted"=> $notify_on_accepted,
        "origin"=> [
          "contactNumber"=> $contact_number,
          "contactEmail"=> $contact_email,
          "contactName"=> $contact_name,
          "locationId"=> "2",
        ],
        "destination"=> [
          "contactNumber"=> $order['phone'],
          "contactEmail"=> $order['email'],
          "contactName"=> $order['name'],
          "locationId"=> $order['destination_locationId'],
        ],
        "items"=> [
          [
            "id"=> "1",
            "name"=> "voucher",
            "value"=> "0.00",
            "compartmentSize"=> 2,
            "weight"=> $order['total_weight'],
          ]
        ]
      ], JSON_THROW_ON_ERROR);
    } catch (JsonException $e) {
      // Handle the exception if JSON encoding fails
      $this->loggerChannelFactory->get('commerce_boxnow')->error('Error encoding JSON: ' . $e->getMessage());
      return FALSE;
    }

    // Make the API request
    try {
      $headers = [
        'Content-Type' => 'application/json',
        'Authorization' => 'Bearer ' . $token,
      ];
      $url = $api_url . self::REQUEST_DELIVERY_PATH;
      $request = new Request('POST', $url, $headers, $body);
      $res = $this->client->sendAsync($request)->wait();
      // Decode JSON response
      $responseBody = $res->getBody()->getContents();
      if (empty($responseBody)) {
        $this->loggerChannelFactory->get('commerce_boxnow')->error('Empty response body from API. URL: @url.', [
          '@url' => self::REQUEST_DELIVERY_PATH,
        ]);
        return FALSE;
      }
      $responseData = json_decode($responseBody, TRUE);
      // Check if decoding was successful
      if (json_last_error() !== JSON_ERROR_NONE) {
        $this->loggerChannelFactory->get('commerce_boxnow')->error('Error decoding JSON response: ' . json_last_error_msg());
        return FALSE;
      }
      $this->loggerChannelFactory->get('commerce_boxnow')->info('Response Data: ' . print_r($responseData, TRUE));
      $api_response = [];
      $api_response['id'] = $responseData['id'];
      foreach ($responseData['parcels'] as $index => $parcel) {
        $api_response['parcel_id'][$index] = $parcel['id'];
      }

      return $api_response;
    }
    catch (RequestException $e) {
      $this->loggerChannelFactory->get('commerce_boxnow')->error('Request to BoxNow failed. URL: @url, Error: @error', [
        '@url' => self::REQUEST_DELIVERY_PATH,
        '@error' => $e->getMessage(),
      ]);
      return FALSE;
    }
  }

  /**
   * Retrieves the BoxNow shipping configuration.
   *
   * Loads the shipping method configuration for the specified
   * shipping method ID, particularly for BoxNow.
   *
   * @param int $shipping_method_id
   *   The ID of the shipping method configuration to load.
   *
   * @return array|bool
   *   An associative array of the shipping method configuration, or FALSE
   *   if the configuration cannot be loaded.
   */
  public function getBoxNowConfiguration() {
    $shipping_method_id = $this->getBoxNowShippingMethodId();
    if (!$shipping_method_id) {
      return [];
    }
    $shipping_method_storage = \Drupal::service('entity_type.manager')->getStorage('commerce_shipping_method');
    $shipping_method = $shipping_method_storage->load($shipping_method_id);
    if ($shipping_method) {
      $plugin_configuration = $shipping_method->get('plugin')->getValue();

      if (isset($plugin_configuration[0]['target_plugin_configuration'])) {
        $target_plugin_configuration = $plugin_configuration[0]['target_plugin_configuration'];
        return $target_plugin_configuration;
      }
    }
    return FALSE;
  }

  public function getBoxNowShippingMethodId() {
    $plugin_id = 'boxnow_shipping';
    // Load all shipping methods.
    $shipping_methods = ShippingMethod::loadMultiple();

    // Filter to find the shipping method(s) with the specified plugin ID.
    $matching_methods = [];
    foreach ($shipping_methods as $shipping_method) {
      if ($shipping_method->getPlugin()->getPluginId() === $plugin_id) {
        $matching_methods[] = $shipping_method->id();
      }
    }
    return $matching_methods[0];
  }
}
