<?php

namespace Drupal\commerce_boxnow\EventSubscriber;

use Drupal\commerce_boxnow\CommerceBoxNowService;
use Drupal\commerce_order\Entity\OrderInterface;
use Drupal\commerce_shipping\Entity\ShipmentInterface;
use Drupal\Core\Messenger\MessengerInterface;
use Drupal\state_machine\Event\WorkflowTransitionEvent;
use Psr\Log\LoggerInterface;
use Drupal\Core\Logger\LoggerChannelFactoryInterface;
use Symfony\Component\EventDispatcher\EventSubscriberInterface;

/**
 * Class OrderPlaceSubscriber.
 */
class OrderPlaceSubscriber implements EventSubscriberInterface {
  /**
   * The BoxNow service.
   *
   * @var \Drupal\commerce_boxnow\CommerceBoxNowService
   */
  protected $boxNowService;

  /**
   * The Messenger service.
   *
   * @var \Drupal\Core\Messenger\MessengerInterface
   */
  protected $messenger;

  /**
   * The Logger service.
   *
   * @var \Drupal\Core\Logger\LoggerChannelFactoryInterface
   */
  protected $loggerChannelFactory;

  /**
   * Constructs an OrderPlaceSubscriber object.
   *
   * @param \Drupal\commerce_boxnow\CommerceBoxNowService $boxNowService
   *   The BoxNow service.
   * @param \Drupal\Core\Messenger\MessengerInterface $messenger
   *   The Messenger service.
   * @param \Drupal\Core\Logger\LoggerChannelFactoryInterface $loggerChannelFactory
   *   The Logger service.
   */
  public function __construct(CommerceBoxNowService $boxNowService, MessengerInterface $messenger, LoggerChannelFactoryInterface $loggerChannelFactory) {
    $this->boxNowService = $boxNowService;
    $this->messenger = $messenger;
    $this->loggerChannelFactory = $loggerChannelFactory;
  }

  /**
   * Processes the order fulfillment and requests a BoxNow delivery.
   *
   * @param \Drupal\state_machine\Event\WorkflowTransitionEvent $event
   *   The workflow transition event containing the order entity.
   */
  public function onPlaceHandler(WorkflowTransitionEvent $event) {
    /** @var \Drupal\commerce_order\Entity\OrderInterface $order_entity */
    $order_entity = $event->getEntity();
    // Check for valid order entity.
    if (!$order_entity instanceof OrderInterface) {
      $this->loggerChannelFactory->get('commerce_boxnow')->error('Invalid order entity in place handler.');
      return NULL;
    }

    // Check if shipment exists.
    $shipments = $order_entity->get('shipments')->referencedEntities();
    if (empty($shipments)) {
      return NULL;
    }

    $shipment = $shipments[0];
    // Ensure the shipping method is BoxNow.
    if ($shipment->getShippingMethod()->getPlugin()->getPluginId() !== 'boxnow_shipping') {
      return NULL;
    }

    //@todo-mk Check for shipping profile.
    $shipping_profile_entity = $shipment->getShippingProfile();
    if (!$shipping_profile_entity) {
      $this->loggerChannelFactory->get('commerce_boxnow')->error('Error in shipping profile data, order_id : ' . $order_entity->id());
      return NULL;
    }

    // Prepare order data for BoxNow API request.
    $given_name = $shipping_profile_entity->get('address')->given_name;
    $family_name = $shipping_profile_entity->get('address')->family_name;
    $order = [
      'number' => $order_entity->getOrderNumber(),
      'invoice_value' => $order_entity->getTotalPrice()?->getNumber(),
      'phone' => $shipment->getData('boxnow_telephone'),
      'email' => $order_entity->getEmail(),
      'name' => $given_name . ' ' . $family_name,
      'destination_locationId' => $shipment->getData('locker_id'),
      'total_weight' => 0,
    ];

    // Request delivery from BoxNow.
    $result = $this->boxNowService->requestDelivery($order);

    // Check if request was successful.
    if (!isset($result['id']) && !isset($result['parcel_id'])) {
      $this->loggerChannelFactory->get('commerce_boxnow')->error('Error requesting BOX NOW delivery in order placement, order_id : ' . $order_entity->id());
      return NULL;
    }

    // Save BoxNow request data to shipment.
    if ($shipment instanceof ShipmentInterface) {
      $shipment->setData('request_id', $result['id']);
      $shipment->setData('parcel_id', $result['parcel_id'][0]);
      $shipment->save();
    }
  }

  /**
   * {@inheritdoc}
   */
  public static function getSubscribedEvents() {
    $events['commerce_order.place.post_transition'] = ['onPlaceHandler'];
    return $events;
  }

}


