<?php

namespace Drupal\commerce_boxnow\Plugin\Commerce\CheckoutPane;

use Drupal\commerce_checkout\Annotation\CommerceCheckoutPane;
use Drupal\commerce_checkout\Plugin\Commerce\CheckoutPane\CheckoutPaneBase;
use Drupal\commerce_shipping\Entity\ShipmentInterface;
use Drupal\commerce_shipping\Entity\ShippingMethod;
use Drupal\Core\Form\FormStateInterface;
use Drupal\Core\Render\Markup;

/**
 * Provides a custom BOX NOW shipping information pane.
 *
 * @CommerceCheckoutPane(
 *   id = "commerce_boxnow_shipping_information",
 *   label = @Translation("BOX NOW Shipping Information"),
 *   wrapper_element = "fieldset",
 *   default_step = "order_information",
 * )
 */
class BoxNowShippingInformation extends CheckoutPaneBase {

  /**
   * {@inheritdoc}
   */
  public function buildPaneForm(array $pane_form, FormStateInterface $form_state, array &$complete_form) {
    $complete_form['#attached']['library'][] = 'commerce_boxnow/commerce_boxnow_widget';

    // Fetch the shipment entity.
    $shipment = $complete_form['shipping_information']['shipments'][0]['#shipment'] ?? NULL;
    if (!$shipment) {
      return $pane_form;
    }

    // Add BoxNow-specific fields.
    $this->addBoxNowFields($pane_form, $complete_form);

    /** @var \Drupal\commerce_boxnow\CommerceBoxNowService $boxnow_service */
    $boxnow_service = \Drupal::service('commerce_boxnow.service');
    $boxnow_config = $boxnow_service->getBoxNowConfiguration();
    if (empty($boxnow_config)) {
      \Drupal::logger('commerce_boxnow')->error('BOX NOW configuration not found.');
      \Drupal::messenger()->addError(t('The BOX NOW shipping method is not properly configured. Please contact the site administrator.'));
      return $pane_form;
    }
    $partner_id = $boxnow_config['partner_id'];
    $complete_form['#attached']['drupalSettings']['boxNow']['partnerId'] = $partner_id;

    // Check the shipping method plugin ID.
    $shipment_plugin_id = $shipment->getShippingMethod()?->getPlugin()->getPluginId();
    $input = $form_state->getUserInput();
    $selected_shipping_method_id = $input['shipping_information']['shipments'][0]['shipping_method'][0] ?? NULL;

    if ($selected_shipping_method_id) {
      // Resolve shipping method plugin ID from user input.
      $parts = explode('--', $selected_shipping_method_id);
      $shipping_method = ShippingMethod::load($parts[0]);
      $shipment_plugin_id = $shipping_method?->getPlugin()->getPluginId();
    }

    // Hide BoxNow fields if the shipping method is not BoxNow.
    if ($shipment_plugin_id !== 'boxnow_shipping') {
      $pane_form['boxnow']['#access'] = FALSE;
    }

    return $pane_form;
  }

  /**
   * {@inheritdoc}
   */
  public function validatePaneForm(array &$pane_form, FormStateInterface $form_state, array &$complete_form) {
//    if (empty($form_state->getValue(['commerce_boxnow_shipping_information', 'boxnow', 'boxnow_locker_id']))) {
//      $form_state->setErrorByName('boxnow_locker_id', t('Please select a BOX NOW locker.'));
//    }
//    if (empty($form_state->getValue(['commerce_boxnow_shipping_information', 'boxnow', 'boxnow_locker_address']))) {
//      $form_state->setErrorByName('boxnow_locker_address', t('Locker address is required. Please choose a BOX NOW locker.'));
//    }
//    if (empty($form_state->getValue(['commerce_boxnow_shipping_information', 'boxnow', 'boxnow_locker_postal_code']))) {
//      $form_state->setErrorByName('boxnow_locker_postal_code', t('Locker postal code is required. Please choose a BOX NOW locker.'));
//    }
  }

  /**
   * {@inheritdoc}
   */
  public function submitPaneForm(array &$pane_form, FormStateInterface $form_state, array &$complete_form) {
    $shipment = $complete_form['shipping_information']['shipments'][0]['#shipment'] ?? NULL;

    if ($shipment instanceof ShipmentInterface) {
      $plugin_id = $shipment->getShippingMethod()?->getPlugin()->getPluginId();

      if ($plugin_id === 'boxnow_shipping') {
        // Save BoxNow-specific locker data.
        $shipment->setData('locker_id', $form_state->getValue(['commerce_boxnow_shipping_information', 'boxnow', 'boxnow_locker_id']));
        $shipment->setData('locker_address', $form_state->getValue(['commerce_boxnow_shipping_information', 'boxnow', 'boxnow_locker_address']));
        $shipment->setData('locker_postal_code', $form_state->getValue(['commerce_boxnow_shipping_information', 'boxnow', 'boxnow_locker_postal_code']));
        $shipment->setData('boxnow_telephone', $form_state->getValue(['commerce_boxnow_shipping_information', 'boxnow', 'boxnow_telephone']));

      } else {
        // Unset BoxNow-specific data if the shipping method is not BoxNow.
        $shipment->unsetData('locker_id');
        $shipment->unsetData('locker_address');
        $shipment->unsetData('locker_postal_code');
        $shipment->unsetData('boxnow_telephone');
      }

      $shipment->save();
    }
  }

  /**
   * Adds BoxNow-specific fields to the checkout pane form.
   */
  private function addBoxNowFields(&$pane_form, &$complete_form) {
    $shipment = $complete_form['shipping_information']['shipments'][0]['#shipment'] ?? NULL;

    $pane_form['boxnow'] = [
      '#type' => 'container',
    ];

    // Button to open the BoxNow map widget
    $pane_form['boxnow']['boxnow_widget_locker_selector'] = [
      '#type' => 'markup',
      '#markup' => Markup::create('<a href="javascript:;" class="boxnow-map-widget-button">' . t("Choose a BOX NOW locker.") . '</a><div id="boxnowmap"></div>'),
    ];

    $pane_form['boxnow']['boxnow_locker_id'] = [
      '#type' => 'hidden',
      '#title' => t('Locker ID'),
      '#default_value' => $shipment?->getData('locker_id') ?? '',
      '#required' => TRUE,
      '#attributes' => [
        'id' => 'boxnow_locker_id',
      ],
    ];

    $pane_form['boxnow']['boxnow_locker_address'] = [
      '#type' => 'textfield',
      '#title' => t('Locker Address'),
      '#default_value' => $shipment?->getData('locker_address') ?? '',
      '#required' => TRUE,
      '#attributes' => [
        'id' => 'boxnow_locker_address',
        'readonly' => 'readonly',
      ],
    ];

    $pane_form['boxnow']['boxnow_locker_postal_code'] = [
      '#type' => 'textfield',
      '#title' => t('Locker Postal Code'),
      '#default_value' => $shipment?->getData('locker_postal_code') ?? '',
      '#required' => TRUE,
      '#attributes' => [
        'id' => 'boxnow_locker_postal_code',
        'readonly' => 'readonly',
      ],
    ];

    $pane_form['boxnow']['boxnow_telephone'] = [
      '#type' => 'textfield',
      '#title' => t('Telephone'),
      '#default_value' => $shipment?->getData('boxnow_telephone') ?? '',
      '#required' => TRUE,
      '#attributes' => [
        'id' => 'boxnow_telephone',
      ],
      '#description' => t('Please enter your phone number starting with country code (e.g., +306969696969).'),
    ];
  }
}
