<?php

namespace Drupal\commerce_boxnow\Plugin\Commerce\ShippingMethod;

use Drupal\commerce_shipping\Annotation\CommerceShippingMethod;
use Drupal\commerce_shipping\Plugin\Commerce\ShippingMethod\FlatRate;
use Drupal\commerce_shipping\PackageTypeManagerInterface;
use Drupal\commerce_shipping\ShippingService;
use Drupal\Core\Form\FormStateInterface;
use Drupal\state_machine\WorkflowManagerInterface;

/**
 * Provides the Commerce BoxNow shipping method.
 *
 * @CommerceShippingMethod(
 *   id = "boxnow_shipping",
 *   label = @Translation("BOXNOW Shipping")
 * )
 */
class CommerceBoxNowShipping extends FlatRate {
  /**
   * Constructs a new BoxNowShipping object.
   *
   * @param array $configuration
   *   A configuration array containing information about the plugin instance.
   * @param string $plugin_id
   *   The plugin_id for the plugin instance.
   * @param mixed $plugin_definition
   *   The plugin implementation definition.
   * @param \Drupal\commerce_shipping\PackageTypeManagerInterface $package_type_manager
   *   The package type manager.
   * @param \Drupal\state_machine\WorkflowManagerInterface $workflow_manager
   *   The workflow manager.
   */
  public function __construct(array $configuration, $plugin_id, $plugin_definition, PackageTypeManagerInterface $package_type_manager, WorkflowManagerInterface $workflow_manager) {
    parent::__construct($configuration, $plugin_id, $plugin_definition, $package_type_manager, $workflow_manager);
    $this->services['default'] = new ShippingService('default', $this->configuration['rate_label']);
  }

  /**
   * {@inheritdoc}
   */
  public function defaultConfiguration() {
    return [
        'api_url' => '',
        'client_id' => '',
        'client_secret' => '',
        'partner_id' => '',
        'notify_on_accepted' => '',
        'contact_number' => '',
        'contact_email' => '',
        'contact_name' => '',
      ] + parent::defaultConfiguration(); // Merging with parent default configuration.
  }

  /**
   * {@inheritdoc}
   */
  public function buildConfigurationForm(array $form, FormStateInterface $form_state) {
    $form = parent::buildConfigurationForm($form, $form_state);
    $form['api_url'] = [
      '#type' => 'textfield',
      '#title' => $this->t('API URL'),
      '#default_value' => $this->configuration['api_url'],
      '#required' => TRUE,
    ];
    $form['client_id'] = [
      '#type' => 'textfield',
      '#title' => $this->t('Client ID'),
      '#default_value' => $this->configuration['client_id'],
      '#required' => TRUE,
    ];
    $form['client_secret'] = [
      '#type' => 'textfield',
      '#title' => $this->t('Client Secret'),
      '#default_value' => $this->configuration['client_secret'],
      '#required' => TRUE,
    ];
    $form['partner_id'] = [
      '#type' => 'textfield',
      '#title' => $this->t('Partner ID'),
      '#default_value' => $this->configuration['partner_id'],
      '#required' => TRUE,
    ];
    $form['notify_on_accepted'] = [
      '#type' => 'textfield',
      '#title' => $this->t('Notify On Accepted'),
      '#default_value' => $this->configuration['notify_on_accepted'],
      '#required' => TRUE,
    ];
    $form['contact_number'] = [
      '#type' => 'textfield',
      '#title' => $this->t('Contact Number'),
      '#default_value' => $this->configuration['contact_number'],
      '#required' => TRUE,
    ];
    $form['contact_email'] = [
      '#type' => 'textfield',
      '#title' => $this->t('Contact Email'),
      '#default_value' => $this->configuration['contact_email'],
      '#required' => TRUE,
    ];
    $form['contact_name'] = [
      '#type' => 'textfield',
      '#title' => $this->t('Contact Name'),
      '#default_value' => $this->configuration['contact_name'],
      '#required' => TRUE,
    ];
    return $form;
  }

  /**
   * {@inheritdoc}
   */
  public function validateConfigurationForm(array &$form, FormStateInterface $form_state) {
    parent::validateConfigurationForm($form, $form_state);
    // Validate emails
    $notify_email = $form_state->getValue('plugin')[0]['target_plugin_configuration']['boxnow_shipping']['notify_on_accepted'];
    $contact_email = $form_state->getValue('plugin')[0]['target_plugin_configuration']['boxnow_shipping']['contact_email'];
    if (!filter_var($notify_email, FILTER_VALIDATE_EMAIL)) {
      $form_state->setErrorByName('notify_on_accepted', $this->t('Please provide a valid notify email address.'));
    }
    if (!filter_var($contact_email, FILTER_VALIDATE_EMAIL)) {
      $form_state->setErrorByName('contact_email', $this->t('Please provide a valid contact email address.'));
    }
  }

  /**
   * {@inheritdoc}
   */
  public function submitConfigurationForm(array &$form, FormStateInterface $form_state) {
    parent::submitConfigurationForm($form, $form_state);

    if (!$form_state->getErrors()) {
      $values = $form_state->getValue($form['#parents']);
      $this->configuration['api_url'] = $values['api_url'];
      $this->configuration['client_id'] = $values['client_id'];
      $this->configuration['client_secret'] = $values['client_secret'];
      $this->configuration['partner_id'] = $values['partner_id'];
      $this->configuration['notify_on_accepted'] = $values['notify_on_accepted'];
      $this->configuration['contact_number'] = $values['contact_number'];
      $this->configuration['contact_email'] = $values['contact_email'];
      $this->configuration['contact_name'] = $values['contact_name'];
    }
  }

}
